"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.rgbaToHex = exports.randomInt = exports.toStringSafe = exports.caseInsensitiveEqual = exports.uuid = exports.toPascalCase = exports.arrayExcept = exports.except = exports.unionInPlace = exports.atob = exports.getHighResolutionTimestamp = exports.quartile = exports.arrayAvg = exports.arraySum = exports.sortAscending = exports.PromiseHelpers = exports.assertNotNaN = exports.assertIsNumber = exports.assertIsBoolean = exports.assertIsString = exports.assertNumElems = exports.assertNotEmpty = exports.assertNotNull = exports.range = exports.deleteUninitializedProperties = exports.assertNever = exports.sum = exports.takeWhile = exports.first = exports.hostIsVisualStudio = exports.HostKind = exports.allDebugTypes = exports.DebugType = void 0;
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const lodash_1 = require("lodash");
const uuid_1 = require("uuid");
var DebugType;
(function (DebugType) {
    DebugType["CppDbg"] = "cppdbg";
    DebugType["CSpy"] = "cspy";
    DebugType["Arm"] = "embedded-debug";
    DebugType["CortexDebug"] = "cortex-debug";
})(DebugType = exports.DebugType || (exports.DebugType = {}));
exports.allDebugTypes = Object.values(DebugType);
var HostKind;
(function (HostKind) {
    HostKind["VSCode"] = "VSCode";
    HostKind["VisualStudio"] = "VisualStudio";
    HostKind["Other"] = "Other";
})(HostKind = exports.HostKind || (exports.HostKind = {}));
function hostIsVisualStudio(hostKind) {
    return hostKind === HostKind.VisualStudio;
}
exports.hostIsVisualStudio = hostIsVisualStudio;
function first(xs) {
    if (xs && xs.length > 0) {
        return xs[0];
    }
    return undefined;
}
exports.first = first;
function takeWhile(xs, pred) {
    const ret = [];
    for (const x of xs) {
        if (pred(x)) {
            ret.push(x);
            continue;
        }
        break;
    }
    return ret;
}
exports.takeWhile = takeWhile;
function sum(xs) {
    return xs.reduce((a, b) => a + b, 0);
}
exports.sum = sum;
/** Call in default or else clause to detect non-exhaustive checks at compile time. Will never execute. */
function assertNever(_) {
    throw new Error("This should be unreachable.");
}
exports.assertNever = assertNever;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function deleteUninitializedProperties(obj) {
    if (typeof obj !== "object") {
        return;
    }
    for (const prop in obj) {
        if (obj.hasOwnProperty(prop)) {
            if (obj[prop] === undefined) {
                delete obj[prop];
            }
            else {
                deleteUninitializedProperties(obj[prop]);
            }
        }
    }
}
exports.deleteUninitializedProperties = deleteUninitializedProperties;
/** Return increasing array of numbers [start, stop) with optional step. */
function range(start, stop, step = 1) {
    const ret = [];
    for (let i = start; i < stop; i += step) {
        ret.push(i);
    }
    return ret;
}
exports.range = range;
function assertNotNull(x, msg = ""
// eslint-disable-next-line @rushstack/no-new-null
) {
    if (x === null || x === undefined) {
        throw new Error(`Assert failed: unexpected null. ${msg}`);
    }
}
exports.assertNotNull = assertNotNull;
function assertNotEmpty(xs, msg = "") {
    if (xs.length < 1) {
        throw new Error(`Assert failed: unexpected empty array. ${msg}`);
    }
}
exports.assertNotEmpty = assertNotEmpty;
function assertNumElems(xs, num, msg = "") {
    if (xs.length !== num) {
        throw new Error(`Assert failed: expected array to have length ${num} but was ${xs.length}. ${msg}`);
    }
}
exports.assertNumElems = assertNumElems;
function assertIsString(x, msg = "") {
    if (!lodash_1.default.isString(x)) {
        throw new Error(`Assert failed: expected ${x} to be a string. ${msg}`);
    }
}
exports.assertIsString = assertIsString;
function assertIsBoolean(x, msg = "") {
    if (!(typeof x === "boolean")) {
        throw new Error(`Assert failed: expected ${x} to be a boolean. ${msg}`);
    }
}
exports.assertIsBoolean = assertIsBoolean;
function assertIsNumber(x, msg = "") {
    if (!(typeof x === "number")) {
        throw new Error(`Assert failed: expected ${x} to be a number. ${msg}`);
    }
}
exports.assertIsNumber = assertIsNumber;
// There's no NaN type in TypeScript, so we can't use a type assertion.
// https://github.com/Microsoft/TypeScript/issues/28682
function assertNotNaN(x, msg = "") {
    if (isNaN(x)) {
        throw new Error(`Assert failed: unexpected NaN. ${msg}`);
    }
}
exports.assertNotNaN = assertNotNaN;
var PromiseHelpers;
(function (PromiseHelpers) {
    function resolveAfter(ms, value) {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        return new Promise((resolve, reject) => {
            setTimeout(() => {
                // If value is undefined, then either undefined is an expected return
                // type from this promise, or Promise<void> is expected. In either case
                // it should be safe to resolve with undefined.
                resolve(value);
            }, ms);
        });
    }
    PromiseHelpers.resolveAfter = resolveAfter;
})(PromiseHelpers = exports.PromiseHelpers || (exports.PromiseHelpers = {}));
function sortAscending(arr) {
    return arr.sort((a, b) => a - b);
}
exports.sortAscending = sortAscending;
function arraySum(arr) {
    return arr.reduce((a, b) => a + b, 0);
}
exports.arraySum = arraySum;
function arrayAvg(arr) {
    return arraySum(arr) / arr.length;
}
exports.arrayAvg = arrayAvg;
function quartile(arr, quartilePercentage) {
    const sorted = sortAscending(arr);
    const position = Math.floor((sorted.length - 1) * quartilePercentage);
    return sorted[position];
}
exports.quartile = quartile;
function getHighResolutionTimestamp() {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        /* eslint-disable @typescript-eslint/no-explicit-any */
        const performance = 
        // In a browser context "performance" is available globally.
        (_a = globalThis.performance) !== null && _a !== void 0 ? _a : 
        // @ts-ignore: no types for perf_hooks
        // In Node "performance" is in the "perf_hooks" module.
        (_b = ((yield Promise.resolve().then(() => require("perf_hooks"))))) === null || _b === void 0 ? void 0 : _b.performance;
        /* eslint-enable @typescript-eslint/no-explicit-any */
        assertNotNull(performance, "Could not find a high resolution time source.");
        return performance.now();
    });
}
exports.getHighResolutionTimestamp = getHighResolutionTimestamp;
/* eslint-disable @typescript-eslint/no-explicit-any */
exports.atob = 
// In a browser context "atob" is available globally.
(_a = globalThis.atob) !== null && _a !== void 0 ? _a : 
// In Node polyfill using Buffer.
((data) => globalThis.Buffer.from(data, "base64").toString("binary"));
/* eslint-enable @typescript-eslint/no-explicit-any */
function unionInPlace(set, other) {
    other.forEach((x) => set.add(x));
    return set;
}
exports.unionInPlace = unionInPlace;
function except(set, other) {
    return new Set(Array.from(set).filter((x) => !other.has(x)));
}
exports.except = except;
function arrayExcept(array, other) {
    const set = new Set(other);
    return array.filter((x) => !set.has(x));
}
exports.arrayExcept = arrayExcept;
function toPascalCase(text) {
    return text[0].toUpperCase() + text.slice(1);
}
exports.toPascalCase = toPascalCase;
function uuid() {
    return (0, uuid_1.v4)();
}
exports.uuid = uuid;
/**
 * Compare strings in case-insensitive fashion.
 * @param s1 First string.
 * @param s2 Second string.
 * @param locale Optional locale for comparision. If not supplied, comparison will be culture-invariant.
 * @returns true iff `s1` and `s2` are case-insensitive equal.
 */
function caseInsensitiveEqual(s1, s2, locale) {
    return locale
        ? s1.toLocaleLowerCase(locale) === s2.toLocaleLowerCase(locale)
        : s1.toLowerCase() === s2.toLowerCase();
}
exports.caseInsensitiveEqual = caseInsensitiveEqual;
/**
 * Returns a string representation of the given value. Strings are returned
 * as-is, while objects are converted to a JSON representation. Circular
 * references are replaced with a "[circular]" string value.
 * @param value Value to convert to a string.
 * @returns The converted string.
 */
function toStringSafe(value) {
    if (typeof value === "string")
        return value;
    // Based on https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Errors/Cyclic_object_value#examples
    const seen = new WeakSet();
    function replacer(_, value) {
        if (typeof value === "object" && value !== null) {
            if (seen.has(value))
                return "[circular]";
            seen.add(value);
        }
        return value;
    }
    return JSON.stringify(value, replacer);
}
exports.toStringSafe = toStringSafe;
/**
 * Return a randomly sampled integer within the given range.
 *
 * @param min Minimum value, defaults to 0.
 * @param max Maximum value, defaults to Number.MAX_SAFE_INTEGER.
 * @returns A randomly sampled integer within the range [min, max). Max is excluded.
 */
function randomInt(min = 0, max = Number.MAX_SAFE_INTEGER) {
    return Math.floor(Math.random() * (max - min) + min);
}
exports.randomInt = randomInt;
/**
 * Return the hex representation of an `rgb()` or `rgba()` CSS color. If parsing fails, returns `undefined`.
 *
 * @param rgba A CSS color string in `rgb()` or `rgba()` form.
 * @returns If alpha value is supplied, an 8-digit hex string with form `#xxxxxxxx`.
 * If alpha value is not supplied, a 6-digit hex string with form `#xxxxxx`. If parsing fails, `undefined`.
 */
function rgbaToHex(rgba) {
    rgba = rgba.replace(/\s+/g, "");
    const regex = /rgba?\((?<red>\d+),(?<green>\d+),(?<blue>\d+)(,(?<alpha>\S+))?\)/;
    const result = rgba.match(regex);
    if (!(result === null || result === void 0 ? void 0 : result.groups)) {
        return undefined;
    }
    const { red, green, blue, alpha } = result.groups;
    const parts = [red, green, blue].map((s) => parseInt(s));
    if (parts.some((p) => isNaN(p) || p < 0 || p > 255)) {
        return undefined;
    }
    if (alpha !== undefined) {
        const alphaFloat = alpha.endsWith("%")
            ? parseFloat(alpha.slice(0, -1)) / 100
            : parseFloat(alpha);
        if (isNaN(alphaFloat) || alphaFloat > 1 || alphaFloat < 0) {
            return undefined;
        }
        parts.push(Math.round(alphaFloat * 255));
    }
    return `#${parts.map((p) => p.toString(16).padStart(2, "0")).join("")}`;
}
exports.rgbaToHex = rgbaToHex;
